import 'dart:core';
import 'package:toolbox_indicator_test/api.dart';
import 'package:toolbox_indicator_test/function.dart';
import 'package:toolbox_indicator_test/variables.dart';
import 'dart:convert';
//import 'package:toolbox_indicator_test/creation.dart';
import 'package:geiger_localstorage/geiger_localstorage.dart';
import 'dart:io';
import 'package:toolbox_indicator_test/listener.dart';
import 'package:flutter_test/flutter_test.dart';

void main() async {

  final StorageController controller =
  GenericController('testOwner', DummyMapper('testOwner'));
  //await ExtendedTimestamp.initializeTimestamp(controller);


  Storage.controller = controller;

  // all tests related to updates of nodes and values
  await updateTests(controller);


  //await init();

  await initPlugin();
  await setValueListener();
  await checkExistNodes();

  await getRecommendation();
  await getRisk();

//  if(Storage.getValuePath.isNotEmpty){

  print(Storage.getValuePath.length);

  await getMetrics(Types.users, Storage.userUUID);
  await updateScoreNode(Types.users, Types.geigerScoreUser);
  await updateRecommendationNode(Types.users);

  await getMetrics(Types.devices, Storage.deviceUUID);
  await updateScoreNode(Types.devices, Types.geigerScoreDevice);
  await updateRecommendationNode(Types.devices);

  await getDevices();
  await getEmployees();
  await crtAggregateScore();
  await updateScoreNode(Types.users, Types.geigerScoreAggregate);

  updateScoreMSENode();

//  }else{
//  print('=======Waiting sensors Values');
//  //Listener.sensorsValue.getNumEvents(1,500);
//
//  var bodys = '{"geigerValue": "32.5", "minValue": "0", "cUUID": "5a6487c7-c1c5-4f28-995e-8805e45bf1a3", "maxValue": "100", "valueType": "double", "mUUID": "123e4567-e89b-42d3-a456-556642440sdk-m01", '
//  '"threatsImpact": "80efffaf-98a1-4e0a-8f5e-gr89388352ph,High;80efffaf-98a1-4e0a-8f5e-gr89388354sp,High;80efffaf-98a1-4e0a-8f5e-th89388365it,High;80efffaf-98a1-4e0a-8f5e-gr89388350ma,Medium;80efffaf-98a1-4e0a-8f5e-gr89388356db,Medium"}';
//  var a = json.decode(bodys.toString());
//
//  await createUserData(a);
//
//  var cbodys = '{"geigerValue": "90.1", "minValue": "0", "cUUID": "5a6487c7-c1c5-4f28-995e-8805e45bf1a3", "maxValue": "100", "valueType": "double", "mUUID": "123e4567-e89b-42d3-a456-556642440sdk-m01", '
//  '"threatsImpact": "80efffaf-98a1-4e0a-8f5e-gr89388352ph,High;80efffaf-98a1-4e0a-8f5e-gr89388354sp,High;80efffaf-98a1-4e0a-8f5e-th89388365it,High;80efffaf-98a1-4e0a-8f5e-gr89388350ma,Medium;80efffaf-98a1-4e0a-8f5e-gr89388356db,Medium"}';
//  var aa = json.decode(cbodys.toString());
//
//  await createDeviceData(aa);

//  await Listener.sensorsValue.getNumEvents(2,1000).whenComplete(() async => await updateData(Listener.sensorsValue.events.iterator));

//  }


}

Future<void> updateTests(final StorageController controller) async {
  group('controller update tests', () {
    test('Owner Update On Node', () async {
      print('## Testing controller in UNKNOWN');
      await controller.add(NodeImpl(':testNodeOwner1', 'testOwner'));
      Node node = await controller.get(':testNodeOwner1');
      expect(node.owner, 'testOwner');
    });

    test('Storage Node Create', () async {
      await controller.add(NodeImpl(':StorageNodeCreate1', 'testOwner'));
      // fetch stored node
      Node storedNode = await controller.get(':StorageNodeCreate1');

      // check results
      expect(storedNode.owner, 'testOwner');
      expect(storedNode.name, 'StorageNodeCreate1');
      expect(storedNode.path, ':StorageNodeCreate1');
      expect(storedNode.visibility, Visibility.red);
    });

    test('Test Storage Node Add', () async {
      await controller.add(NodeImpl('parent1', 'testOwner', ':'));
      await controller.add(NodeImpl('name2', 'testOwner', ':parent1'));

      // get the record
      Node storedNode = await controller.get(':parent1:name2');

      // check results
      expect(storedNode.owner, 'testOwner');
      expect(storedNode.name, 'name2');
      expect(storedNode.path, ':parent1:name2');
      expect(storedNode.visibility, Visibility.red);
    });

    // depends on correct functionality of the StorageController.create() function
    test('test storage node update', () async {
      // create original node
      await controller.add(NodeImpl(':nodeUpdateTest', 'testOwner'));

      // updated Node with different visibility children
      Node node = NodeImpl(
          'testNode1', 'testOwner', ':nodeUpdateTest', Visibility.green);
      await controller.add(node);
      expect(node.owner, 'testOwner');

      Node sn =
      NodeImpl('testChild1', 'testOwner', ':nodeUpdateTest:testNode1');
      await controller.add(sn);
      node.visibility = Visibility.red;
      await node.addChild(sn);

      // update with node from above
      await controller.update(node);

      // get the record
      Node storedNode = await controller.get(':nodeUpdateTest:testNode1');

      // check results
      expect(storedNode.owner, 'testOwner');
      expect(storedNode.name, 'testNode1');
      expect(storedNode.path, ':nodeUpdateTest:testNode1');
      expect(await storedNode.getChildNodesCsv(), 'testChild1');
      expect(storedNode.visibility, Visibility.red);
    });

    test('add node with missing parent', () {
      expect(
              () async => await controller
              .add(NodeImpl('testNode1', 'testOwner', ':nodeUpdateTest2')),
          throwsA(TypeMatcher<StorageException>()));
    });

    test('create new node', () async {
      List<Node> n = <Node>[
        NodeImpl(':nodeCreateTest', 'testOwner'),
        NodeImpl(':nodeCreateTest:testNode1', 'testOwner')
      ];

      for (final Node tn in n) {
        print('## creating new node ${tn.path} (parent of ${tn.parentPath})');
        await controller.add(tn);
      }

      // add a value
      print('## adding value');
      await controller.addValue(
          ':nodeCreateTest:testNode1', NodeValueImpl('key1', 'valueFirst'));

      // update value
      print('## updating value');
      NodeValue value2 = NodeValueImpl('key1', 'valueSecond');
      await controller.updateValue(':nodeCreateTest:testNode1', value2);

      // get the record
      print('## testing updated value');
      Node n2 = await controller.get(':nodeCreateTest:testNode1');
      expect(
          (await n2.getValue(value2.key) ??
              NodeValueImpl(value2.key, 'INVALID'))
              .value,
          value2.value);

      print(
          '## testing removal of node ${n.first} with child nodes (${n.last.path})');

      try {
        await controller.delete(n[0].path);
        fail('Parental node was deleted despite a child node exists');
      } on StorageException catch (e) {
        print('## got expected exception when deleting parent node ($e)');
      } catch (e) {
        fail('got exception of wrong type ${e.toString()}');
      }

      for (final Node tn in List.from(n.reversed)) {
        print('## removing node ${tn.path} (parent of ${tn.parentPath})');
        await controller.delete(tn.path);
      }
    });
  });
}


//Future init() async {
//  await createLocal();
//  await createThreats();
//  await createPlugin();
//  await createProfiles();
//  await createRecommendations();
//  await createEnterprise();
//
//
//  //print(await Storage.controller!.dump(':Global'));
//  //await createEnterpriseDevices();
//  //await createEnterpriseUsers();
//  //await createMetricData();
//
//}


